@@ -7,6 +7,14 @@
 #
 # Changelog:
 #
+# 2022-08-16:
+#
+# * Added `has_ansi()`. This checks that cli is installed and that
+#   `cli::num_ansi_colors()` is greater than 1.
+#
+# * `col_` and `style_` functions now consistently return bare strings.
+#
+#
 # 2022-05-23:
 #
 # * Added compat for `style_hyperlink()`.
@@ -90,49 +98,49 @@ ansi_alert  <- function() col_yellow(symbol_alert())
 #' @param x A string.
 #'
 #' @noRd
-col_black              <- function(x) if (.rlang_cli_has_ansi()) cli::col_black(x) else x
-col_blue               <- function(x) if (.rlang_cli_has_ansi()) cli::col_blue(x) else x
-col_cyan               <- function(x) if (.rlang_cli_has_ansi()) cli::col_cyan(x) else x
-col_green              <- function(x) if (.rlang_cli_has_ansi()) cli::col_green(x) else x
-col_magenta            <- function(x) if (.rlang_cli_has_ansi()) cli::col_magenta(x) else x
-col_red                <- function(x) if (.rlang_cli_has_ansi()) cli::col_red(x) else x
-col_white              <- function(x) if (.rlang_cli_has_ansi()) cli::col_white(x) else x
-col_yellow             <- function(x) if (.rlang_cli_has_ansi()) cli::col_yellow(x) else x
-col_grey               <- function(x) if (.rlang_cli_has_ansi()) cli::col_grey(x) else x
-col_silver             <- function(x) if (.rlang_cli_has_ansi()) cli::col_silver(x) else x
-col_none               <- function(x) if (.rlang_cli_has_ansi()) cli::col_none(x) else x
+col_black              <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_black(x)) else x
+col_blue               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_blue(x)) else x
+col_cyan               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_cyan(x)) else x
+col_green              <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_green(x)) else x
+col_magenta            <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_magenta(x)) else x
+col_red                <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_red(x)) else x
+col_white              <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_white(x)) else x
+col_yellow             <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_yellow(x)) else x
+col_grey               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_grey(x)) else x
+col_silver             <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_silver(x)) else x
+col_none               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::col_none(x)) else x
 
-bg_black               <- function(x) if (.rlang_cli_has_ansi()) cli::bg_black(x) else x
-bg_blue                <- function(x) if (.rlang_cli_has_ansi()) cli::bg_blue(x) else x
-bg_cyan                <- function(x) if (.rlang_cli_has_ansi()) cli::bg_cyan(x) else x
-bg_green               <- function(x) if (.rlang_cli_has_ansi()) cli::bg_green(x) else x
-bg_magenta             <- function(x) if (.rlang_cli_has_ansi()) cli::bg_magenta(x) else x
-bg_red                 <- function(x) if (.rlang_cli_has_ansi()) cli::bg_red(x) else x
-bg_white               <- function(x) if (.rlang_cli_has_ansi()) cli::bg_white(x) else x
-bg_yellow              <- function(x) if (.rlang_cli_has_ansi()) cli::bg_yellow(x) else x
-bg_none                <- function(x) if (.rlang_cli_has_ansi()) cli::bg_none(x) else x
+bg_black               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_black(x)) else x
+bg_blue                <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_blue(x)) else x
+bg_cyan                <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_cyan(x)) else x
+bg_green               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_green(x)) else x
+bg_magenta             <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_magenta(x)) else x
+bg_red                 <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_red(x)) else x
+bg_white               <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_white(x)) else x
+bg_yellow              <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_yellow(x)) else x
+bg_none                <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::bg_none(x)) else x
 
-style_dim              <- function(x) if (.rlang_cli_has_ansi()) cli::style_dim(x) else x
-style_blurred          <- function(x) if (.rlang_cli_has_ansi()) cli::style_blurred(x) else x
-style_bold             <- function(x) if (.rlang_cli_has_ansi()) cli::style_bold(x) else x
-style_hidden           <- function(x) if (.rlang_cli_has_ansi()) cli::style_hidden(x) else x
-style_inverse          <- function(x) if (.rlang_cli_has_ansi()) cli::style_inverse(x) else x
-style_italic           <- function(x) if (.rlang_cli_has_ansi()) cli::style_italic(x) else x
-style_strikethrough    <- function(x) if (.rlang_cli_has_ansi()) cli::style_strikethrough(x) else x
-style_underline        <- function(x) if (.rlang_cli_has_ansi()) cli::style_underline(x) else x
+style_dim              <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_dim(x)) else x
+style_blurred          <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_blurred(x)) else x
+style_bold             <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_bold(x)) else x
+style_hidden           <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_hidden(x)) else x
+style_inverse          <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_inverse(x)) else x
+style_italic           <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_italic(x)) else x
+style_strikethrough    <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_strikethrough(x)) else x
+style_underline        <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_underline(x)) else x
 
-style_no_dim           <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_dim(x) else x
-style_no_blurred       <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_blurred(x) else x
-style_no_bold          <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_bold(x) else x
-style_no_hidden        <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_hidden(x) else x
-style_no_inverse       <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_inverse(x) else x
-style_no_italic        <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_italic(x) else x
-style_no_strikethrough <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_strikethrough(x) else x
-style_no_underline     <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_underline(x) else x
+style_no_dim           <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_dim(x)) else x
+style_no_blurred       <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_blurred(x)) else x
+style_no_bold          <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_bold(x)) else x
+style_no_hidden        <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_hidden(x)) else x
+style_no_inverse       <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_inverse(x)) else x
+style_no_italic        <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_italic(x)) else x
+style_no_strikethrough <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_strikethrough(x)) else x
+style_no_underline     <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_underline(x)) else x
 
-style_reset            <- function(x) if (.rlang_cli_has_ansi()) cli::style_reset(x) else x
-style_no_colour        <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_color(x) else x
-style_no_bg_colour     <- function(x) if (.rlang_cli_has_ansi()) cli::style_no_bg_color(x) else x
+style_reset            <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_reset(x)) else x
+style_no_colour        <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_color(x)) else x
+style_no_bg_colour     <- function(x) if (.rlang_cli_has_cli()) .rlang_cli_unstructure(cli::style_no_bg_color(x)) else x
 
 CLI_SUPPORT_HYPERLINK <- "2.2.0"
 CLI_SUPPORT_HYPERLINK_PARAMS <- "3.1.1"
@@ -363,7 +371,7 @@ format_message <- function(x) {
   out
 }
 
-.rlang_cli_has_ansi <- function() {
+has_ansi <- function() {
   .rlang_cli_has_cli() && cli::num_ansi_colors() > 1
 }
 
@@ -451,5 +459,9 @@ cli_escape <- function(x) {
   stop(sprintf("Internal error in rlang shims: Unknown function `%s()`.", fn))
 }
 
+.rlang_cli_unstructure <- function(x) {
+  attributes(x) <- NULL
+  x
+}
 
 # nocov end
