@@ -1,3 +1,7 @@
+options(
+  crayon.enabled = FALSE,
+  cli.unicode = FALSE
+)
 
 opt <- Sys.getenv("rlang_backtrace_on_error")
 if (nzchar(opt)) {
