@@ -35,7 +35,7 @@ test_that("can rethrow from `try_fetch()`", {
     if (chain) {
       try_fetch(f(), error = function(cnd) abort("bar", parent = cnd))
     } else {
-      try_fetch(f(), error = function(cnd) abort("bar", error = cnd))
+      try_fetch(f(), error = function(cnd) abort("bar", parent = NA))
     }
   }
 
@@ -79,7 +79,7 @@ test_that("cnd_muffle() returns FALSE if the condition is not mufflable", {
 })
 
 test_that("drop_global_handlers() works and is idempotent", {
-  skip_if_not_installed("base", "4.0")
+  skip_if_not_installed("base", "4.0.0")
 
   code <- '{
     library(testthat)
