@@ -1,5 +1,3 @@
-context("deparse")
-
 test_that("line_push() adds indentation", {
   out <- line_push("foo", "bar", width = 4, indent = 2)
   expect_identical(out, c("foo", "  bar"))
@@ -32,8 +30,9 @@ test_that("line_push() handles the nchar(line) == boundary case", {
 })
 
 test_that("line_push() strips ANSI codes before computing overflow", {
-  if (!has_crayon()) {
-    skip("test needs crayon")
+  local_options(cli.num_colors = 8L)
+  if (!has_ansi()) {
+    skip("test needs cli")
   }
   expect_identical(length(line_push("foo", open_blue(), width = 3L)), 2L)
   expect_identical(length(line_push("foo", open_blue(), width = 3L, has_colour = TRUE)), 1L)
@@ -182,11 +181,15 @@ test_that("call_deparse() handles multi-line arguments", {
 })
 
 test_that("call_deparse() delimits CAR when needed", {
-  call <- expr((!!quote(function() x + 1))())
+  fn_call <- quote(function() x + 1)
+  call <- expr((!!fn_call)())
   expect_identical(call_deparse(call), "(function() x + 1)()")
 
-  # Only equal because of the extra parentheses
-  expect_equal(parse_expr(expr_deparse(call)), call)
+  roundtrip <- parse_expr(expr_deparse(call))
+  exp <- call2(call("(", fn_call))
+
+  # Zap srcref to work around https://github.com/r-lib/waldo/issues/59
+  expect_equal(zap_srcref(roundtrip), zap_srcref(exp))
 
   call <- expr((!!quote(f + g))(x))
   expect_identical(call_deparse(call), "`+`(f, g)(x)")
@@ -253,11 +256,27 @@ test_that("literal lists are deparsed", {
   expect_identical(sexp_deparse(list(TRUE, b = 2L, 3, d = "4", as.raw(5))), "<list: TRUE, b = 2L, 3, d = \"4\", <raw: 05>>")
 })
 
-test_that("long vectors are truncated", {
+test_that("long vectors are truncated by default", {
   expect_identical(sexp_deparse(1:10), "<int: 1L, 2L, 3L, 4L, 5L, ...>")
   expect_identical(sexp_deparse(as.list(1:10)), "<list: 1L, 2L, 3L, 4L, 5L, ...>")
 })
 
+test_that("long vectors are truncated when max_elements = 0L", {
+  lines <- new_lines(max_elements = 0L)
+  expect_identical(sexp_deparse(1:10, lines), "<int: ...>")
+
+  lines <- new_lines(max_elements = 0L)
+  expect_identical(sexp_deparse(as.list(1:10), lines), "<list: ...>")
+})
+
+test_that("long vectors are not truncated when max_elements = NULL", {
+  lines <- new_lines(max_elements = NULL)
+  expect_identical(sexp_deparse(1:10, lines), "<int: 1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, 10L>")
+
+  lines <- new_lines(max_elements = NULL)
+  expect_identical(sexp_deparse(as.list(1:10), lines), "<list: 1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, 10L>")
+})
+
 test_that("other objects are deparsed with base deparser", {
   expect_identical_(sexp_deparse(expr(foo((!!base::list)(1, 2)))), "foo(.Primitive(\"list\")(1, 2))")
   expect_identical_(sexp_deparse(expr(foo((!!base::`if`)(1, 2)))), "foo(.Primitive(\"if\")(1, 2))")
@@ -407,7 +426,6 @@ test_that("non-syntactic symbols are deparsed with backticks", {
 })
 
 test_that("symbols with unicode are deparsed consistently (#691)", {
-  skip_unless_utf8()
   skip_if(getRversion() < "3.2")
 
   expect_identical(expr_text(sym("\u00e2a")), "\u00e2a")
@@ -482,3 +500,226 @@ test_that("`next` and `break` are deparsed", {
   expect_equal(expr_deparse(quote({ next; (break) })), c("{", "  next",  "  (break)", "}"))
   expect_equal(expr_deparse(quote(a <- next <- break)), c("a <- next <- break"))
 })
+
+test_that("double colon is never wrapped (#1072)", {
+  expect_identical(
+    expr_deparse(quote(some.very.long::construct), width = 20),
+    "some.very.long::construct"
+  )
+  expect_identical(
+    expr_deparse(quote(id_function <- base::identity), width = 15),
+    c(
+      "id_function <-",
+      "  base::identity"
+    )
+  )
+  expect_identical(
+    expr_deparse(quote(id_fun <- base::identity), width = 20),
+    "id_fun <- base::identity"
+  )
+})
+
+test_that("triple colon is never wrapped (#1072)", {
+  expect_identical(
+    expr_deparse(quote(some.very.long:::construct), width = 20),
+    "some.very.long:::construct"
+  )
+  expect_identical(
+    expr_deparse(quote(id_function <- base:::identity), width = 15),
+    c(
+      "id_function <-",
+      "  base:::identity"
+    )
+  )
+  expect_identical(
+    expr_deparse(quote(id_fun <- base:::identity), width = 20),
+    "id_fun <- base:::identity"
+  )
+})
+
+test_that("backslashes in strings are properly escaped (#1160)", {
+  expect_equal(
+    expr_deparse(sym("a\\b")),
+    "`a\\\\b`"
+  )
+
+  # Escaping ensures this roundtrip
+  expect_equal(
+    parse_expr(expr_deparse(sym("a\\b"))),
+    sym("a\\b")
+  )
+
+  # Argument names
+  expect_equal(
+    expr_deparse(quote(c("a\\b" = "c\\d"))),
+    "c(`a\\\\b` = \"c\\\\d\")"
+  )
+
+  # Vector names
+  expect_equal(
+    expr_deparse(c("a\\b" = "c\\d")),
+    "<chr: a\\\\b = \"c\\\\d\">"
+  )
+  expect_equal(
+    expr_deparse(list("a\\b" = "c\\d")),
+    "<list: a\\\\b = \"c\\\\d\">"
+  )
+})
+
+test_that("formulas are deparsed (#1169)", {
+  # Evaluated formulas are treated as objects
+  expect_equal(
+    expr_deparse(~foo),
+    "<formula>"
+  )
+
+  # Unevaluated formulas with a symbol have no space
+  expect_equal(
+    expr_deparse(quote(~foo)),
+    "~foo"
+  )
+
+  # Unevaluated formulas with expressions have a space
+  expect_equal(
+    expr_deparse(quote(~+foo)),
+    "~ +foo"
+  )
+  expect_equal(
+    expr_deparse(quote(~foo())),
+    "~ foo()"
+  )
+})
+
+test_that("matrices and arrays are formatted (#383)", {
+  mat <- matrix(1:3)
+  expect_equal(as_label(mat), "<int[,1]>")
+  expect_equal(expr_deparse(mat), "<int[,1]: 1L, 2L, 3L>")
+
+  mat2 <- matrix(1:4, 2)
+  expect_equal(as_label(mat2), "<int[,2]>")
+  expect_equal(expr_deparse(mat2), "<int[,2]: 1L, 2L, 3L, 4L>")
+
+  arr <- array(1:3, c(1, 1, 3))
+  expect_equal(as_label(arr), "<int[,1,3]>")
+  expect_equal(expr_deparse(arr), "<int[,1,3]: 1L, 2L, 3L>")
+})
+
+test_that("infix operators are labelled (#956, r-lib/testthat#1432)", {
+  expect_equal(
+    as_label(quote({ 1; 2} + 3)),
+    "... + 3"
+  )
+
+  expect_equal(
+    as_label(quote(`+`(1, 2, 3))),
+    "`+`(1, 2, 3)"
+  )
+
+  expect_equal(
+    as_label(quote(
+      arg + arg + arg + arg + arg + arg + arg + arg + arg + arg + arg + arg
+    )),
+    "... + arg"
+  )
+
+  expect_equal(
+    as_label(quote(X[key1 == "val1" & key2 == "val2"]$key3 & foobarbaz(foobarbaz(foobarbaz(foobarbaz(foobarbaz(foobarbaz(foobarbaz())))))))),
+    "X[key1 == \"val1\" & key2 == \"val2\"]$key3 & ..."
+  )
+
+  expect_equal(
+    as_label(quote(X[key1 == "val1"]$key3 & foobarbaz(foobarbaz()))),
+    "X[key1 == \"val1\"]$key3 & foobarbaz(foobarbaz())"
+  )
+
+  # This fits in 60 characters so we don't need to truncate it
+  expect_equal(
+    as_label(quote(nchar(chr, type = "bytes", allowNA = TRUE) == 1)),
+    "nchar(chr, type = \"bytes\", allowNA = TRUE) == 1"
+  )
+
+  # This fits into 60 characters if we truncate either side,
+  # so we don't need to shorten both of them
+  expect_equal(
+    as_label(quote(very_long_expression[with(subsetting), -1] -
+                     another_very_long_expression[with(subsetting), -1]
+    )),
+    "very_long_expression[with(subsetting), -1] - ..."
+  )
+
+  lhs_perfect_fit <- sym(paste(rep("a", 56), collapse = ""))
+  lhs_no_fit <- sym(paste(rep("a", 57), collapse = ""))
+
+  expect_equal(
+    as_label(expr(!!lhs_perfect_fit + 1)),
+    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + 1"
+  )
+  expect_equal(
+    as_label(expr(!!lhs_perfect_fit + 10)),
+    "... + 10"
+  )
+
+  expect_equal(
+    as_label(expr(1 + !!lhs_perfect_fit)),
+    "1 + aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
+  )
+  expect_equal(
+    as_label(expr(10 + !!lhs_perfect_fit)),
+    "10 + ..."
+  )
+
+  expect_equal(
+    as_label(expr(!!lhs_no_fit + 1)),
+    "... + 1"
+  )
+  expect_equal(
+    as_label(expr(!!lhs_no_fit + !!lhs_no_fit)),
+    "... + ..."
+  )
+})
+
+test_that("binary op without arguments", {
+  expect_equal(
+    expr_deparse(quote(`+`())),
+    "`+`()"
+  )
+  expect_equal(
+    expr_deparse(quote(`$`())),
+    "`$`()"
+  )
+  expect_equal(
+    expr_deparse(quote(`~`())),
+    "`~`()"
+  )
+})
+
+test_that("call_deparse_highlight() handles long lists of arguments (#1456)", {
+  out <- call_deparse_highlight(quote(
+    foo(
+      aaaaaa = aaaaaa,
+      bbbbbb = bbbbbb,
+      cccccc = cccccc,
+      dddddd = dddddd,
+      eeeeee = eeeeee
+    )
+  ), NULL)
+
+  expect_equal(
+    cli::ansi_strip(out),
+    "foo(aaaaaa = aaaaaa, bbbbbb = bbbbbb, cccccc = cccccc, dddddd = dddddd, eeeeee = eeeeee)"
+  )
+})
+
+test_that("call_deparse_highlight() handles multi-line arguments (#1456)", {
+  out <- call_deparse_highlight(quote(
+    fn(arg = {
+      a
+      b
+    })
+  ), NULL)
+
+  expect_equal(
+    cli::ansi_strip(out),
+    "fn(arg = {\n  a\n  b\n})"
+  )
+})
